@@ -7,16 +7,18 @@
  * Tests if copying an image as data uri works.
  */
 
-const SVG_URL = EXAMPLE_URL + "dropmarker.svg";
+const SVG_URL = HTTPS_EXAMPLE_URL + "dropmarker.svg";
 
-add_task(async function() {
-  const { tab, monitor } = await initNetMonitor(CURL_URL, { requestCount: 1 });
+add_task(async function () {
+  const { tab, monitor } = await initNetMonitor(HTTPS_CURL_URL, {
+    requestCount: 1,
+  });
   info("Starting test... ");
 
   const { document } = monitor.panelWin;
 
   const wait = waitForNetworkEvents(monitor, 1);
-  await SpecialPowers.spawn(tab.linkedBrowser, [SVG_URL], async function(url) {
+  await SpecialPowers.spawn(tab.linkedBrowser, [SVG_URL], async function (url) {
     content.wrappedJSObject.performRequest(url);
   });
   await wait;
@@ -31,11 +33,11 @@ add_task(async function() {
   );
 
   await waitForClipboardPromise(
-    function setup() {
-      getContextMenuItem(
+    async function setup() {
+      await selectContextMenuItem(
         monitor,
         "request-list-context-copy-image-as-data-uri"
-      ).click();
+      );
     },
     function check(text) {
       return text.startsWith("data:") && !/undefined/.test(text);
