@@ -91,6 +91,7 @@ const HEADERS_NON_L10N_STRINGS = {
     hostOnly: "HostOnly",
     isHttpOnly: "HttpOnly",
     isSecure: "Secure",
+    partitionKey: "Partition Key",
     path: "Path",
     sameSite: "SameSite",
     uniqueKey: "Unique key",
@@ -167,14 +168,18 @@ class StorageUI {
     this.table.on(TableWidget.EVENTS.CELL_EDIT, this.editItem);
 
     this.sidebar = this._panelDoc.getElementById("storage-sidebar");
+
+    // Set suggested sizes for the xul:splitter's, so that the sidebar doesn't take too much space
+    // in horizontal mode (width) and vertical (height).
     this.sidebar.style.width = "300px";
+    this.sidebar.style.height = "300px";
+
     this.view = new lazy.VariablesView(this.sidebar.firstChild, {
       lazyEmpty: true,
       // ms
       lazyEmptyDelay: 10,
       searchEnabled: true,
       contextMenuId: "variable-view-popup",
-      preventDescriptorModifiers: true,
     });
 
     this.filterItems = this.filterItems.bind(this);
