@@ -16,17 +16,19 @@ var {
 
 const lazy = {};
 
-ChromeUtils.defineESModuleGetters(lazy, {
-  FileUtils: "resource://gre/modules/FileUtils.sys.mjs",
-  NetworkHelper:
-    "resource://devtools/shared/network-observer/NetworkHelper.sys.mjs",
-});
-
-ChromeUtils.defineModuleGetter(
-  lazy,
-  "ObjectUtils",
-  "resource://gre/modules/ObjectUtils.jsm"
-);
+if (!isWorker) {
+  ChromeUtils.defineESModuleGetters(
+    lazy,
+    {
+      DownloadPaths: "resource://gre/modules/DownloadPaths.sys.mjs",
+      FileUtils: "resource://gre/modules/FileUtils.sys.mjs",
+      NetworkHelper:
+        "resource://devtools/shared/network-observer/NetworkHelper.sys.mjs",
+      ObjectUtils: "resource://gre/modules/ObjectUtils.sys.mjs",
+    },
+    { global: "contextual" }
+  );
+}
 
 // Native getters which are considered to be side effect free.
 ChromeUtils.defineLazyGetter(lazy, "sideEffectFreeGetters", () => {
@@ -133,7 +135,7 @@ exports.waitForTime = function (delay) {
 };
 
 /**
- * Like XPCOMUtils.defineLazyGetter, but with a |this| sensitive getter that
+ * Like ChromeUtils.defineLazyGetter, but with a |this| sensitive getter that
  * allows the lazy getter to be defined on a prototype and work correctly with
  * instances.
  *
@@ -464,7 +466,8 @@ DevToolsUtils.defineLazyGetter(this, "AppConstants", () => {
     return {};
   }
   return ChromeUtils.importESModule(
-    "resource://gre/modules/AppConstants.sys.mjs"
+    "resource://gre/modules/AppConstants.sys.mjs",
+    { global: "contextual" }
   ).AppConstants;
 });
 
@@ -513,7 +516,9 @@ Object.defineProperty(exports, "assert", {
 });
 
 DevToolsUtils.defineLazyGetter(this, "NetUtil", () => {
-  return ChromeUtils.import("resource://gre/modules/NetUtil.jsm").NetUtil;
+  return ChromeUtils.importESModule("resource://gre/modules/NetUtil.sys.mjs", {
+    global: "contextual",
+  }).NetUtil;
 });
 
 /**
@@ -532,6 +537,7 @@ DevToolsUtils.defineLazyGetter(this, "NetUtil", () => {
  *        - principal: the principal to use, if omitted, the request is loaded
  *                     with a content principal corresponding to the url being
  *                     loaded, using the origin attributes of the window, if any.
+ *        - headers: extra headers
  *        - cacheKey: when loading from cache, use this key to retrieve a cache
  *                    specific to a given SHEntry. (Allows loading POST
  *                    requests from cache)
@@ -554,6 +560,7 @@ function mainThreadFetch(
     window: null,
     charset: null,
     principal: null,
+    headers: null,
     cacheKey: 0,
   }
 ) {
@@ -587,6 +594,12 @@ function mainThreadFetch(
       }
     }
 
+    if (aOptions.headers && channel instanceof Ci.nsIHttpChannel) {
+      for (const h in aOptions.headers) {
+        channel.setRequestHeader(h, aOptions.headers[h], /* aMerge = */ false);
+      }
+    }
+
     if (aOptions.window) {
       // Respect private browsing.
       channel.loadGroup = aOptions.window.docShell.QueryInterface(
@@ -618,7 +631,7 @@ function mainThreadFetch(
             // If there was a real stream error, we would have already rejected above.
             resolve({
               content: "",
-              contentType: "text/plan",
+              contentType: "text/plain",
             });
             return;
           }
@@ -841,13 +854,17 @@ exports.saveAs = async function (
   try {
     returnFile = await exports.showSaveFileDialog(
       parentWindow,
-      fileName,
+      lazy.DownloadPaths.sanitize(fileName),
       filters
     );
   } catch (ex) {
     return null;
   }
 
+  // Sanitize the filename again in case the user renamed the file to use a
+  // vulnerable extension.
+  returnFile.leafName = lazy.DownloadPaths.sanitize(returnFile.leafName);
+
   await IOUtils.write(returnFile.path, dataArray, {
     tmpPath: returnFile.path + ".tmp",
   });
@@ -881,7 +898,7 @@ exports.showSaveFileDialog = function (
     fp.defaultString = suggestedFilename;
   }
 
-  fp.init(parentWindow, null, fp.modeSave);
+  fp.init(parentWindow.browsingContext, null, fp.modeSave);
   if (Array.isArray(filters) && filters.length) {
     for (const { pattern, label } of filters) {
       fp.appendFilter(label, pattern);
@@ -955,6 +972,12 @@ function callPropertyOnObject(object, name, ...args) {
     throw new Error("Not a callable object.");
   }
 
+  if (value.script !== undefined) {
+    throw new Error(
+      "The property isn't a native function and will execute code in the debuggee"
+    );
+  }
+
   // Call the property.
   const result = value.call(object, ...args);
   if (result === null) {
