@@ -6,11 +6,11 @@
 
 const {
   TYPES: { LOCAL_STORAGE },
-} = require("devtools/shared/commands/resource/resource-command");
+} = require("resource://devtools/shared/commands/resource/resource-command.js");
 
-const { Front, types } = require("devtools/shared/protocol.js");
+const { Front, types } = require("resource://devtools/shared/protocol.js");
 
-module.exports = function({ resource, watcherFront, targetFront }) {
+module.exports = function ({ resource, watcherFront, targetFront }) {
   if (!(resource instanceof Front) && watcherFront) {
     // instantiate front for local storage
     resource = types.getType("localStorage").read(resource, targetFront);
