@@ -2,19 +2,17 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 8.12.1-1_22
+es5id: 8.12.1-1_30
 description: >
-    Properties - [[HasOwnProperty]] (literal own getter/setter
-    property)
+    Properties - [[HasOwnProperty]] (non-configurable, non-enumerable
+    own setter property)
 ---*/
 
-var o = {
-  get foo() {
-    return 42;
-  },
-  set foo(x) {;
+var o = {};
+Object.defineProperty(o, "foo", {
+  set: function() {;
   }
-};
+});
 
 assert(o.hasOwnProperty("foo"), 'o.hasOwnProperty("foo") !== true');
 
