@@ -4,115 +4,118 @@
 
 // Original author: ekr@rtfm.com
 
-#include <iostream>
-
 #include "logging.h"
 #include "nss.h"
+#include "ssl.h"
 
+#include "api/audio/builtin_audio_processing_builder.h"
+#include "api/audio_codecs/builtin_audio_decoder_factory.h"
+#include "api/environment/environment_factory.h"
+#include "api/scoped_refptr.h"
 #include "AudioSegment.h"
-#include "AudioStreamTrack.h"
+#include "Canonicals.h"
+#include "modules/audio_device/include/fake_audio_device.h"
+#include "modules/audio_mixer/audio_mixer_impl.h"
+#include "modules/audio_processing/include/audio_processing.h"
 #include "mozilla/Mutex.h"
 #include "mozilla/RefPtr.h"
+#include "mozilla/SpinEventLoopUntil.h"
+#include "MediaConduitInterface.h"
 #include "MediaPipeline.h"
 #include "MediaPipelineFilter.h"
 #include "MediaTrackGraph.h"
 #include "MediaTrackListener.h"
-#include "MediaStreamTrack.h"
-#include "transportflow.h"
-#include "transportlayerloopback.h"
-#include "transportlayerdtls.h"
-#include "transportlayersrtp.h"
-#include "mozilla/SyncRunnable.h"
+#include "TaskQueueWrapper.h"
 #include "mtransport_test_utils.h"
 #include "SharedBuffer.h"
 #include "MediaTransportHandler.h"
+#include "WebrtcCallWrapper.h"
+#include "WebrtcEnvironmentWrapper.h"
+#include "PeerConnectionCtx.h"
 
 #define GTEST_HAS_RTTI 0
 #include "gtest/gtest.h"
 
 using namespace mozilla;
-MOZ_MTLOG_MODULE("mediapipeline")
+MOZ_MTLOG_MODULE("transportbridge")
 
 static MtransportTestUtils* test_utils;
 
 namespace {
+class MainAsCurrent : public TaskQueueWrapper<DeletionPolicy::NonBlocking> {
+ public:
+  MainAsCurrent()
+      : TaskQueueWrapper(
+            TaskQueue::Create(do_AddRef(GetMainThreadSerialEventTarget()),
+                              "MainAsCurrentTaskQueue"),
+            "MainAsCurrent"_ns),
+        mSetter(this) {
+    MOZ_RELEASE_ASSERT(NS_IsMainThread());
+  }
 
-class FakeAudioTrack : public mozilla::ProcessedMediaTrack {
+  ~MainAsCurrent() = default;
+
+ private:
+  CurrentTaskQueueSetter mSetter;
+};
+
+class FakeAudioTrack : public ProcessedMediaTrack {
  public:
   FakeAudioTrack()
       : ProcessedMediaTrack(44100, MediaSegment::AUDIO, nullptr),
         mMutex("Fake AudioTrack") {
-    Resume();
-  }
-
-  void Destroy() override {
-    MOZ_ASSERT(!mMainThreadDestroyed);
-    mMainThreadDestroyed = true;
-    Suspend();
-  }
-
-  void QueueSetAutoend(bool) override {}
-
-  void Suspend() override {
-    mozilla::MutexAutoLock lock(mMutex);
-    if (mSuspended) {
-      return;
-    }
-    mSuspended = true;
-    mTimer->Cancel();
-    mTimer = nullptr;
-  }
-
-  void Resume() override {
-    mozilla::MutexAutoLock lock(mMutex);
-    if (!mSuspended) {
-      return;
-    }
-    mSuspended = false;
     NS_NewTimerWithFuncCallback(
         getter_AddRefs(mTimer), FakeAudioTrackGenerateData, this, 20,
         nsITimer::TYPE_REPEATING_SLACK,
         "FakeAudioTrack::FakeAudioTrackGenerateData", test_utils->sts_target());
   }
 
+  void Destroy() override {
+    MutexAutoLock lock(mMutex);
+    MOZ_RELEASE_ASSERT(!mMainThreadDestroyed);
+    mMainThreadDestroyed = true;
+    mTimer->Cancel();
+    mTimer = nullptr;
+  }
+
+  void QueueSetAutoend(bool) override {}
+
   void AddListener(MediaTrackListener* aListener) override {
-    mozilla::MutexAutoLock lock(mMutex);
-    MOZ_ASSERT(!mListener);
+    MutexAutoLock lock(mMutex);
+    MOZ_RELEASE_ASSERT(!mListener);
     mListener = aListener;
   }
 
   RefPtr<GenericPromise> RemoveListener(
       MediaTrackListener* aListener) override {
-    mozilla::MutexAutoLock lock(mMutex);
-    MOZ_ASSERT(mListener == aListener);
+    MutexAutoLock lock(mMutex);
+    MOZ_RELEASE_ASSERT(mListener == aListener);
     mListener = nullptr;
     return GenericPromise::CreateAndResolve(true, __func__);
   }
 
   void ProcessInput(GraphTime aFrom, GraphTime aTo, uint32_t aFlags) override {}
 
+  uint32_t NumberOfChannels() const override { return NUM_CHANNELS; }
+
  private:
-  mozilla::Mutex mMutex;
+  Mutex mMutex MOZ_UNANNOTATED;
   MediaTrackListener* mListener = nullptr;
-  bool mSuspended = true;
   nsCOMPtr<nsITimer> mTimer;
   int mCount = 0;
 
+  static const int AUDIO_BUFFER_SIZE = 1600;
+  static const int NUM_CHANNELS = 2;
   static void FakeAudioTrackGenerateData(nsITimer* timer, void* closure) {
     auto t = static_cast<FakeAudioTrack*>(closure);
-    const int AUDIO_BUFFER_SIZE = 1600;
-    const int NUM_CHANNELS = 2;
-
-    mozilla::MutexAutoLock lock(t->mMutex);
-    if (t->mSuspended) {
+    MutexAutoLock lock(t->mMutex);
+    if (t->mMainThreadDestroyed) {
       return;
     }
-
     CheckedInt<size_t> bufferSize(sizeof(int16_t));
     bufferSize *= NUM_CHANNELS;
     bufferSize *= AUDIO_BUFFER_SIZE;
-    RefPtr<mozilla::SharedBuffer> samples =
-        mozilla::SharedBuffer::Create(bufferSize);
+    RefPtr<SharedBuffer> samples = SharedBuffer::Create(bufferSize);
     int16_t* data = reinterpret_cast<int16_t*>(samples->Data());
     for (int i = 0; i < (AUDIO_BUFFER_SIZE * NUM_CHANNELS); i++) {
       // saw tooth audio sample
@@ -120,7 +123,7 @@ class FakeAudioTrack : public mozilla::ProcessedMediaTrack {
       t->mCount++;
     }
 
-    mozilla::AudioSegment segment;
+    AudioSegment segment;
     AutoTArray<const int16_t*, 1> channels;
     channels.AppendElement(data);
     segment.AppendFrames(samples.forget(), channels, AUDIO_BUFFER_SIZE,
@@ -132,13 +135,21 @@ class FakeAudioTrack : public mozilla::ProcessedMediaTrack {
   }
 };
 
+template <typename Function>
+void RunOnSts(Function&& aFunction) {
+  MOZ_ALWAYS_SUCCEEDS(test_utils->SyncDispatchToSTS(
+      NS_NewRunnableFunction(__func__, [&] { aFunction(); })));
+}
+
 class LoopbackTransport : public MediaTransportHandler {
  public:
   LoopbackTransport() : MediaTransportHandler(nullptr) {
-    SetState("mux", TransportLayer::TS_INIT, false);
-    SetState("mux", TransportLayer::TS_INIT, true);
-    SetState("non-mux", TransportLayer::TS_INIT, false);
-    SetState("non-mux", TransportLayer::TS_INIT, true);
+    RunOnSts([&] {
+      SetState("mux", TransportLayer::TS_INIT);
+      SetRtcpState("mux", TransportLayer::TS_INIT);
+      SetState("non-mux", TransportLayer::TS_INIT);
+      SetRtcpState("non-mux", TransportLayer::TS_INIT);
+    });
   }
 
   static void InitAndConnect(LoopbackTransport& client,
@@ -159,8 +170,9 @@ class LoopbackTransport : public MediaTransportHandler {
   void EnterPrivateMode() override {}
   void ExitPrivateMode() override {}
 
-  nsresult CreateIceCtx(const std::string& aName,
-                        const nsTArray<dom::RTCIceServer>& aIceServers,
+  void CreateIceCtx(const std::string& aName) override {}
+
+  nsresult SetIceConfig(const nsTArray<dom::RTCIceServer>& aIceServers,
                         dom::RTCIceTransportPolicy aIcePolicy) override {
     return NS_OK;
   }
@@ -174,7 +186,7 @@ class LoopbackTransport : public MediaTransportHandler {
   void EnsureProvisionalTransport(const std::string& aTransportId,
                                   const std::string& aLocalUfrag,
                                   const std::string& aLocalPwd,
-                                  size_t aComponentCount) override {}
+                                  int aComponentCount) override {}
 
   void SetTargetForDefaultLocalAddressLookup(const std::string& aTargetIp,
                                              uint16_t aTargetPort) override {}
@@ -215,31 +227,50 @@ class LoopbackTransport : public MediaTransportHandler {
 
   void SendPacket(const std::string& aTransportId,
                   MediaPacket&& aPacket) override {
-    peer_->SignalPacketReceived(aTransportId, aPacket);
+    peer_->LoopbackPacketReceived(aTransportId, aPacket);
   }
 
-  void SetState(const std::string& aTransportId, TransportLayer::State aState,
-                bool aRtcp) {
-    if (aRtcp) {
-      MediaTransportHandler::OnRtcpStateChange(aTransportId, aState);
-    } else {
-      MediaTransportHandler::OnStateChange(aTransportId, aState);
-    }
+  void SetState(const std::string& aTransportId, TransportLayer::State aState) {
+    MediaTransportHandler::OnStateChange(aTransportId, aState);
   }
 
+  void SetRtcpState(const std::string& aTransportId,
+                    TransportLayer::State aState) {
+    MediaTransportHandler::OnRtcpStateChange(aTransportId, aState);
+  }
+
+  void LoopbackPacketReceived(const std::string& aTransportId,
+                              const MediaPacket& aPacket) {
+    if (aPacket.len() && aPacket.type() == MediaPacket::RTCP) {
+      ++rtcp_packets_received_;
+    }
+    SignalPacketReceived(aTransportId, aPacket);
+  }
+
+  int RtcpPacketsReceived() const { return rtcp_packets_received_; }
+
  private:
-  RefPtr<MediaTransportHandler> peer_;
+  RefPtr<LoopbackTransport> peer_;
+  std::atomic<int> rtcp_packets_received_{0};
 };
 
 class TestAgent {
  public:
-  TestAgent()
-      : audio_config_(109, "opus", 48000, 2, false),
-        audio_conduit_(mozilla::AudioSessionConduit::Create(
-            WebRtcCallWrapper::Create(dom::RTCStatsTimestampMaker()),
-            test_utils->sts_target())),
-        audio_pipeline_(),
-        transport_(new LoopbackTransport) {}
+  explicit TestAgent(const RefPtr<WebrtcEnvironmentWrapper>& aEnvWrapper,
+                     const RefPtr<SharedWebrtcState>& aSharedState)
+      : control_(aSharedState->mCallWorkerThread),
+        audio_config_(109, "opus", 48000, 2, false),
+        call_(WebrtcCallWrapper::Create(
+            aEnvWrapper, mozilla::dom::RTCStatsTimestampMaker::Create(),
+            nullptr, aSharedState)),
+        audio_conduit_(
+            AudioSessionConduit::Create(call_, test_utils->sts_target())),
+        transport_(new LoopbackTransport) {
+    Unused << WaitFor(InvokeAsync(call_->mCallThread, __func__, [&] {
+      audio_conduit_->InitControl(&control_);
+      return GenericPromise::CreateAndResolve(true, "TestAgent()");
+    }));
+  }
 
   static void Connect(TestAgent* client, TestAgent* server) {
     LoopbackTransport::InitAndConnect(*client->transport_, *server->transport_);
@@ -247,46 +278,52 @@ class TestAgent {
 
   virtual void CreatePipeline(const std::string& aTransportId) = 0;
 
-  void SetState(const std::string& aTransportId, TransportLayer::State aState,
-                bool aRtcp) {
-    test_utils->sts_target()->Dispatch(
-        WrapRunnable(transport_, &LoopbackTransport::SetState, aTransportId,
-                     aState, aRtcp),
-        nsISerialEventTarget::DISPATCH_SYNC);
+  void SetState_s(const std::string& aTransportId,
+                  TransportLayer::State aState) {
+    transport_->SetState(aTransportId, aState);
   }
 
-  void UpdateTransport(const std::string& aTransportId,
-                       UniquePtr<MediaPipelineFilter>&& aFilter) {
-    test_utils->sts_target()->Dispatch(
-        NS_NewRunnableFunction(__func__,
-                               [pipeline = audio_pipeline_, aTransportId,
-                                filter = std::move(aFilter)]() mutable {
-                                 pipeline->UpdateTransport_s(aTransportId,
-                                                             std::move(filter));
-                               }),
-        nsISerialEventTarget::DISPATCH_SYNC);
+  void SetRtcpState_s(const std::string& aTransportId,
+                      TransportLayer::State aState) {
+    transport_->SetRtcpState(aTransportId, aState);
+  }
+
+  void UpdateTransport_s(const std::string& aTransportId,
+                         UniquePtr<MediaPipelineFilter>&& aFilter) {
+    audio_pipeline_->UpdateTransport_s(aTransportId, std::move(aFilter), false);
   }
 
   void Stop() {
     MOZ_MTLOG(ML_DEBUG, "Stopping");
 
-    if (audio_pipeline_) audio_pipeline_->Stop();
+    control_.Update([](auto& aControl) {
+      aControl.mTransmitting = false;
+      aControl.mReceiving = false;
+    });
   }
 
   void Shutdown_s() { transport_->Shutdown(); }
 
   void Shutdown() {
-    if (audio_pipeline_) audio_pipeline_->Shutdown_m();
+    if (audio_pipeline_) {
+      audio_pipeline_->Shutdown();
+    }
+    if (audio_conduit_) {
+      Unused << WaitFor(audio_conduit_->Shutdown());
+    }
+    if (call_) {
+      call_->Destroy();
+    }
+    if (audio_track_) {
+      audio_track_->Destroy();
+      audio_track_ = nullptr;
+    }
 
-    test_utils->sts_target()->Dispatch(
-        WrapRunnable(this, &TestAgent::Shutdown_s),
-        nsISerialEventTarget::DISPATCH_SYNC);
+    test_utils->SyncDispatchToSTS(WrapRunnable(this, &TestAgent::Shutdown_s));
   }
 
   uint32_t GetRemoteSSRC() {
-    uint32_t res = 0;
-    audio_conduit_->GetRemoteSSRC(&res);
-    return res;
+    return audio_conduit_->GetRemoteSSRC().valueOr(0);
   }
 
   uint32_t GetLocalSSRC() {
@@ -303,29 +340,29 @@ class TestAgent {
 
   int GetAudioRtcpCountSent() { return audio_pipeline_->RtcpPacketsSent(); }
 
-  int GetAudioRtcpCountReceived() {
-    return audio_pipeline_->RtcpPacketsReceived();
-  }
+  int GetAudioRtcpCountReceived() { return transport_->RtcpPacketsReceived(); }
 
  protected:
-  mozilla::AudioCodecConfig audio_config_;
-  RefPtr<mozilla::MediaSessionConduit> audio_conduit_;
+  ConcreteControl control_;
+  AudioCodecConfig audio_config_;
+  RefPtr<WebrtcCallWrapper> call_;
+  RefPtr<AudioSessionConduit> audio_conduit_;
   RefPtr<FakeAudioTrack> audio_track_;
   // TODO(bcampen@mozilla.com): Right now this does not let us test RTCP in
   // both directions; only the sender's RTCP is sent, but the receiver should
   // be sending it too.
-  RefPtr<mozilla::MediaPipeline> audio_pipeline_;
+  RefPtr<MediaPipeline> audio_pipeline_;
   RefPtr<LoopbackTransport> transport_;
 };
 
 class TestAgentSend : public TestAgent {
  public:
-  TestAgentSend() {
-    mozilla::MediaConduitErrorCode err =
-        static_cast<mozilla::AudioSessionConduit*>(audio_conduit_.get())
-            ->ConfigureSendMediaCodec(&audio_config_);
-    EXPECT_EQ(mozilla::kMediaConduitNoError, err);
-
+  explicit TestAgentSend(const RefPtr<WebrtcEnvironmentWrapper>& aEnvWrapper,
+                         const RefPtr<SharedWebrtcState>& aSharedState)
+      : TestAgent(aEnvWrapper, aSharedState) {
+    control_.Update([&](auto& aControl) {
+      aControl.mAudioSendCodec = Some(audio_config_);
+    });
     audio_track_ = new FakeAudioTrack();
   }
 
@@ -333,42 +370,50 @@ class TestAgentSend : public TestAgent {
     std::string test_pc;
 
     RefPtr<MediaPipelineTransmit> audio_pipeline =
-        new mozilla::MediaPipelineTransmit(test_pc, transport_, nullptr,
-                                           test_utils->sts_target(), false,
-                                           audio_conduit_);
-
-    audio_pipeline->SetSendTrack(audio_track_);
-    audio_pipeline->Start();
+        MediaPipelineTransmit::Create(
+            test_pc, transport_, AbstractThread::MainThread(),
+            test_utils->sts_target(), false, audio_conduit_);
+    Unused << WaitFor(InvokeAsync(call_->mCallThread, __func__, [&] {
+      audio_pipeline->InitControl(&control_);
+      return GenericPromise::CreateAndResolve(true, __func__);
+    }));
 
+    audio_pipeline->SetSendTrackOverride(audio_track_);
+    control_.Update([](auto& aControl) { aControl.mTransmitting = true; });
+    audio_pipeline->UpdateTransport_m(aTransportId, nullptr, true);
     audio_pipeline_ = audio_pipeline;
-
-    audio_pipeline_->UpdateTransport_m(aTransportId, nullptr);
   }
 };
 
 class TestAgentReceive : public TestAgent {
  public:
-  TestAgentReceive() {
-    std::vector<UniquePtr<mozilla::AudioCodecConfig>> codecs;
-    codecs.emplace_back(new AudioCodecConfig(audio_config_));
-
-    mozilla::MediaConduitErrorCode err =
-        static_cast<mozilla::AudioSessionConduit*>(audio_conduit_.get())
-            ->ConfigureRecvMediaCodecs(codecs);
-    EXPECT_EQ(mozilla::kMediaConduitNoError, err);
+  explicit TestAgentReceive(const RefPtr<WebrtcEnvironmentWrapper>& aEnvWrapper,
+                            const RefPtr<SharedWebrtcState>& aSharedState)
+      : TestAgent(aEnvWrapper, aSharedState) {
+    control_.Update([&](auto& aControl) {
+      std::vector<AudioCodecConfig> codecs;
+      codecs.push_back(audio_config_);
+      aControl.mAudioRecvCodecs = codecs;
+    });
   }
 
   virtual void CreatePipeline(const std::string& aTransportId) {
     std::string test_pc;
 
-    audio_pipeline_ = new mozilla::MediaPipelineReceiveAudio(
-        test_pc, transport_, nullptr, test_utils->sts_target(),
-        static_cast<mozilla::AudioSessionConduit*>(audio_conduit_.get()),
-        nullptr, PRINCIPAL_HANDLE_NONE);
+    auto audio_pipeline = MakeRefPtr<MediaPipelineReceiveAudio>(
+        test_pc, transport_, AbstractThread::MainThread(),
+        test_utils->sts_target(),
+        static_cast<AudioSessionConduit*>(audio_conduit_.get()), nullptr,
+        TrackingId(), PRINCIPAL_HANDLE_NONE, PrincipalPrivacy::NonPrivate);
+    Unused << WaitFor(InvokeAsync(call_->mCallThread, __func__, [&] {
+      audio_pipeline->InitControl(&control_);
+      return GenericPromise::CreateAndResolve(true, __func__);
+    }));
 
-    audio_pipeline_->Start();
-
-    audio_pipeline_->UpdateTransport_m(aTransportId, std::move(bundle_filter_));
+    control_.Update([](auto& aControl) { aControl.mReceiving = true; });
+    audio_pipeline->UpdateTransport_m(aTransportId, std::move(bundle_filter_),
+                                      true);
+    audio_pipeline_ = audio_pipeline;
   }
 
   void SetBundleFilter(UniquePtr<MediaPipelineFilter>&& filter) {
@@ -377,7 +422,7 @@ class TestAgentReceive : public TestAgent {
 
   void UpdateTransport_s(const std::string& aTransportId,
                          UniquePtr<MediaPipelineFilter>&& filter) {
-    audio_pipeline_->UpdateTransport_s(aTransportId, std::move(filter));
+    audio_pipeline_->UpdateTransport_s(aTransportId, std::move(filter), false);
   }
 
  private:
@@ -390,11 +435,37 @@ void WaitFor(TimeDuration aDuration) {
       NS_NewRunnableFunction(__func__, [&] { done = true; }),
       aDuration.ToMilliseconds());
   SpinEventLoopUntil<ProcessFailureBehavior::IgnoreAndContinue>(
-      [&] { return done; });
+      "WaitFor(TimeDuration aDuration)"_ns, [&] { return done; });
+}
+
+webrtc::AudioState::Config CreateAudioStateConfig(
+    const webrtc::Environment& aEnv) {
+  webrtc::AudioState::Config audio_state_config;
+  audio_state_config.audio_mixer = webrtc::AudioMixerImpl::Create();
+
+  webrtc::BuiltinAudioProcessingBuilder audio_processing_builder;
+  audio_state_config.audio_processing = audio_processing_builder.Build(aEnv);
+  audio_state_config.audio_device_module = new webrtc::FakeAudioDeviceModule();
+  return audio_state_config;
 }
 
 class MediaPipelineTest : public ::testing::Test {
  public:
+  MediaPipelineTest()
+      : main_task_queue_(
+            WrapUnique<TaskQueueWrapper<DeletionPolicy::NonBlocking>>(
+                new MainAsCurrent())),
+        env_wrapper_(WebrtcEnvironmentWrapper::Create(
+            mozilla::dom::RTCStatsTimestampMaker::Create())),
+        shared_state_(MakeAndAddRef<SharedWebrtcState>(
+            AbstractThread::MainThread(),
+            CreateAudioStateConfig(env_wrapper_->Environment()),
+            already_AddRefed(
+                webrtc::CreateBuiltinAudioDecoderFactory().release()),
+            WrapUnique(new webrtc::MozTrialsConfig()))),
+        p1_(env_wrapper_, shared_state_),
+        p2_(env_wrapper_, shared_state_) {}
+
   ~MediaPipelineTest() {
     p1_.Shutdown();
     p2_.Shutdown();
@@ -408,9 +479,8 @@ class MediaPipelineTest : public ::testing::Test {
 
   // Setup transport.
   void InitTransports() {
-    test_utils->sts_target()->Dispatch(
-        WrapRunnableNM(&TestAgent::Connect, &p2_, &p1_),
-        nsISerialEventTarget::DISPATCH_SYNC);
+    test_utils->SyncDispatchToSTS(
+        WrapRunnableNM(&TestAgent::Connect, &p2_, &p1_));
   }
 
   // Verify RTP and RTCP
@@ -436,19 +506,23 @@ class MediaPipelineTest : public ::testing::Test {
     // Set state of transports to CONNECTING. MediaPipeline doesn't really care
     // about this transition, but we're trying to simluate what happens in a
     // real case.
-    p1_.SetState(transportId, TransportLayer::TS_CONNECTING, false);
-    p1_.SetState(transportId, TransportLayer::TS_CONNECTING, true);
-    p2_.SetState(transportId, TransportLayer::TS_CONNECTING, false);
-    p2_.SetState(transportId, TransportLayer::TS_CONNECTING, true);
+    RunOnSts([&] {
+      p1_.SetState_s(transportId, TransportLayer::TS_CONNECTING);
+      p1_.SetRtcpState_s(transportId, TransportLayer::TS_CONNECTING);
+      p2_.SetState_s(transportId, TransportLayer::TS_CONNECTING);
+      p2_.SetRtcpState_s(transportId, TransportLayer::TS_CONNECTING);
+    });
 
     WaitFor(TimeDuration::FromMilliseconds(10));
 
     // Set state of transports to OPEN (ie; connected). This should result in
     // media flowing.
-    p1_.SetState(transportId, TransportLayer::TS_OPEN, false);
-    p1_.SetState(transportId, TransportLayer::TS_OPEN, true);
-    p2_.SetState(transportId, TransportLayer::TS_OPEN, false);
-    p2_.SetState(transportId, TransportLayer::TS_OPEN, true);
+    RunOnSts([&] {
+      p1_.SetState_s(transportId, TransportLayer::TS_OPEN);
+      p1_.SetRtcpState_s(transportId, TransportLayer::TS_OPEN);
+      p2_.SetState_s(transportId, TransportLayer::TS_OPEN);
+      p2_.SetRtcpState_s(transportId, TransportLayer::TS_OPEN);
+    });
 
     if (bundle) {
       WaitFor(TimeDuration::FromMilliseconds(ms_until_filter_update));
@@ -461,7 +535,9 @@ class MediaPipelineTest : public ::testing::Test {
         refinedFilter->AddRemoteSSRC(p1_.GetLocalSSRC());
       }
 
-      p2_.UpdateTransport(transportId, std::move(refinedFilter));
+      RunOnSts([&] {
+        p2_.UpdateTransport_s(transportId, std::move(refinedFilter));
+      });
     }
 
     // wait for some RTP/RTCP tx and rx to happen
@@ -500,6 +576,11 @@ class MediaPipelineTest : public ::testing::Test {
   }
 
  protected:
+  // main_task_queue_ has this type to make sure it goes through Delete() when
+  // we're destroyed.
+  UniquePtr<TaskQueueWrapper<DeletionPolicy::NonBlocking>> main_task_queue_;
+  const RefPtr<WebrtcEnvironmentWrapper> env_wrapper_;
+  const RefPtr<SharedWebrtcState> shared_state_;
   TestAgentSend p1_;
   TestAgentReceive p2_;
 };
@@ -507,13 +588,11 @@ class MediaPipelineTest : public ::testing::Test {
 class MediaPipelineFilterTest : public ::testing::Test {
  public:
   bool Filter(MediaPipelineFilter& filter, uint32_t ssrc, uint8_t payload_type,
-              const mozilla::Maybe<std::string>& mid = mozilla::Nothing()) {
+              const Maybe<std::string>& mid = Nothing()) {
     webrtc::RTPHeader header;
     header.ssrc = ssrc;
     header.payloadType = payload_type;
-    mid.apply([&](const auto& mid) {
-      header.extension.mid.Set(mid.c_str(), mid.length());
-    });
+    mid.apply([&](const auto& mid) { header.extension.mid = mid; });
     return filter.Filter(header);
   }
 };
@@ -532,6 +611,15 @@ TEST_F(MediaPipelineFilterTest, TestSSRCFilter) {
   EXPECT_FALSE(Filter(filter, 556, 110));
 }
 
+TEST_F(MediaPipelineFilterTest, TestSSRCFilterOverridesPayloadTypeFilter) {
+  MediaPipelineFilter filter;
+  filter.AddRemoteSSRC(555);
+  filter.AddUniqueReceivePT(110);
+  // We have a configured ssrc but still need to allow payload type matching.
+  EXPECT_TRUE(Filter(filter, 556, 110));
+  EXPECT_TRUE(Filter(filter, 555, 110));
+}
+
 #define SSRC(ssrc)                                                    \
   ((ssrc >> 24) & 0xFF), ((ssrc >> 16) & 0xFF), ((ssrc >> 8) & 0xFF), \
       (ssrc & 0xFF)
@@ -549,13 +637,26 @@ TEST_F(MediaPipelineFilterTest, TestMidFilter) {
   EXPECT_TRUE(Filter(filter, 16, 110, mid));
   EXPECT_TRUE(Filter(filter, 16, 110));
   EXPECT_FALSE(Filter(filter, 17, 110));
+
+  // The mid filter maintains a set of SSRCs. Adding a new SSRC should work
+  // and still allow previous SSRCs to work. Unrecognized SSRCs should still be
+  // filtered out.
+  EXPECT_TRUE(Filter(filter, 18, 111, mid));
+  EXPECT_TRUE(Filter(filter, 18, 111));
+  EXPECT_TRUE(Filter(filter, 16, 110));
+  EXPECT_FALSE(Filter(filter, 17, 110));
 }
 
 TEST_F(MediaPipelineFilterTest, TestPayloadTypeFilter) {
   MediaPipelineFilter filter;
-  filter.AddUniquePT(110);
+  filter.AddUniqueReceivePT(110);
   EXPECT_TRUE(Filter(filter, 555, 110));
   EXPECT_FALSE(Filter(filter, 556, 111));
+  // Matching based on unique payload type causes us to learn the ssrc.
+  EXPECT_TRUE(Filter(filter, 555, 98));
+  // Once we have learned an SSRC we still need to learn new ones
+  // based on payload type.
+  EXPECT_TRUE(Filter(filter, 557, 110));
 }
 
 TEST_F(MediaPipelineFilterTest, TestSSRCMovedWithMid) {
@@ -577,14 +678,14 @@ TEST_F(MediaPipelineFilterTest, TestRemoteSDPNoSSRCs) {
   MediaPipelineFilter filter;
   const auto mid = Some(std::string("mid0"));
   filter.SetRemoteMediaStreamId(mid);
-  filter.AddUniquePT(111);
+  filter.AddUniqueReceivePT(111);
   EXPECT_TRUE(Filter(filter, 555, 110, mid));
   EXPECT_TRUE(Filter(filter, 555, 110));
 
   // Update but remember binding./
   MediaPipelineFilter filter2;
 
-  filter.Update(filter2);
+  filter.Update(filter2, true);
 
   // Ensure that the old SSRC still works.
   EXPECT_TRUE(Filter(filter, 555, 110));
@@ -592,7 +693,7 @@ TEST_F(MediaPipelineFilterTest, TestRemoteSDPNoSSRCs) {
   // Forget the previous binding
   MediaPipelineFilter filter3;
   filter3.SetRemoteMediaStreamId(Some(std::string("mid1")));
-  filter.Update(filter3);
+  filter.Update(filter3, true);
 
   ASSERT_FALSE(Filter(filter, 555, 110));
 }
