@@ -70,7 +70,9 @@ static MediaControlKey ToMediaControlKey(int aKeyCode) {
 namespace mozilla {
 namespace widget {
 
-bool MediaHardwareKeysEventSourceMac::IsOpened() const { return mEventTap && mEventTapSource; }
+bool MediaHardwareKeysEventSourceMac::IsOpened() const {
+  return mEventTap && mEventTapSource;
+}
 
 bool MediaHardwareKeysEventSourceMac::Open() {
   LOG("Open MediaHardwareKeysEventSourceMac");
@@ -89,22 +91,24 @@ bool MediaHardwareKeysEventSourceMac::StartEventTap() {
   MOZ_ASSERT(!mEventTapSource);
 
   // Add an event tap to intercept the system defined media key events.
-  mEventTap =
-      CGEventTapCreate(kCGSessionEventTap, kCGHeadInsertEventTap, kCGEventTapOptionListenOnly,
-                       CGEventMaskBit(NX_SYSDEFINED), EventTapCallback, this);
+  mEventTap = CGEventTapCreate(
+      kCGSessionEventTap, kCGHeadInsertEventTap, kCGEventTapOptionListenOnly,
+      CGEventMaskBit(NX_SYSDEFINED), EventTapCallback, this);
   if (!mEventTap) {
     LOG("Fail to create event tap");
     return false;
   }
 
-  mEventTapSource = CFMachPortCreateRunLoopSource(kCFAllocatorDefault, mEventTap, 0);
+  mEventTapSource =
+      CFMachPortCreateRunLoopSource(kCFAllocatorDefault, mEventTap, 0);
   if (!mEventTapSource) {
     LOG("Fail to create an event tap source.");
     return false;
   }
 
   LOG("Add an event tap source to current loop");
-  CFRunLoopAddSource(CFRunLoopGetCurrent(), mEventTapSource, kCFRunLoopCommonModes);
+  CFRunLoopAddSource(CFRunLoopGetCurrent(), mEventTapSource,
+                     kCFRunLoopCommonModes);
   return true;
 }
 
@@ -115,18 +119,20 @@ void MediaHardwareKeysEventSourceMac::StopEventTap() {
     mEventTap = nullptr;
   }
   if (mEventTapSource) {
-    CFRunLoopRemoveSource(CFRunLoopGetCurrent(), mEventTapSource, kCFRunLoopCommonModes);
+    CFRunLoopRemoveSource(CFRunLoopGetCurrent(), mEventTapSource,
+                          kCFRunLoopCommonModes);
     CFRelease(mEventTapSource);
     mEventTapSource = nullptr;
   }
 }
 
-CGEventRef MediaHardwareKeysEventSourceMac::EventTapCallback(CGEventTapProxy proxy,
-                                                             CGEventType type, CGEventRef event,
-                                                             void* refcon) {
+CGEventRef MediaHardwareKeysEventSourceMac::EventTapCallback(
+    CGEventTapProxy proxy, CGEventType type, CGEventRef event, void* refcon) {
   // Re-enable event tap when receiving disabled events.
-  MediaHardwareKeysEventSourceMac* source = static_cast<MediaHardwareKeysEventSourceMac*>(refcon);
-  if (type == kCGEventTapDisabledByUserInput || type == kCGEventTapDisabledByTimeout) {
+  MediaHardwareKeysEventSourceMac* source =
+      static_cast<MediaHardwareKeysEventSourceMac*>(refcon);
+  if (type == kCGEventTapDisabledByUserInput ||
+      type == kCGEventTapDisabledByTimeout) {
     MOZ_ASSERT(source->mEventTap);
     CGEventTapEnable(source->mEventTap, true);
     return event;
@@ -151,8 +157,9 @@ CGEventRef MediaHardwareKeysEventSourceMac::EventTapCallback(CGEventTapProxy pro
   // - keyRepeat = (keyFlags & 0x1);
   const NSInteger data1 = [nsEvent data1];
   int keyCode = (data1 & 0xFFFF0000) >> 16;
-  if (keyCode != NX_KEYTYPE_PLAY && keyCode != NX_KEYTYPE_NEXT && keyCode != NX_KEYTYPE_PREVIOUS &&
-      keyCode != NX_KEYTYPE_FAST && keyCode != NX_KEYTYPE_REWIND) {
+  if (keyCode != NX_KEYTYPE_PLAY && keyCode != NX_KEYTYPE_NEXT &&
+      keyCode != NX_KEYTYPE_PREVIOUS && keyCode != NX_KEYTYPE_FAST &&
+      keyCode != NX_KEYTYPE_REWIND) {
     return event;
   }
 
@@ -173,7 +180,8 @@ CGEventRef MediaHardwareKeysEventSourceMac::EventTapCallback(CGEventTapProxy pro
   }
 
   LOG2("Get media key %s", source, ToMediaControlKeyStr(keyCode));
-  for (auto iter = source->mListeners.begin(); iter != source->mListeners.end(); ++iter) {
+  for (auto iter = source->mListeners.begin(); iter != source->mListeners.end();
+       ++iter) {
     (*iter)->OnActionPerformed(MediaControlAction(ToMediaControlKey(keyCode)));
   }
   return event;
