@@ -14,6 +14,8 @@
 #include "nsBaseWidget.h"
 #include "nsCocoaUtils.h"
 #include "nsTouchBar.h"
+#include "ViewRegion.h"
+#include "mozView.h"
 #include <dlfcn.h>
 #include <queue>
 
@@ -24,6 +26,10 @@ class nsMenuBarX;
 
 namespace mozilla {
 enum class NativeKeyBindingsType : uint8_t;
+class VibrancyManager;
+namespace widget {
+class TextInputHandler;
+}
 }  // namespace mozilla
 
 // NSWindow subclass that is the base class for all of our own window classes.
@@ -57,6 +63,8 @@ enum class NativeKeyBindingsType : uint8_t;
 - (NSMutableDictionary*)exportState;
 - (void)setDrawsContentsIntoWindowFrame:(BOOL)aState;
 - (BOOL)drawsContentsIntoWindowFrame;
+- (NSRect)childViewFrameRectForCurrentBounds;
+- (void)updateChildViewFrameRect;
 
 // These two methods are like contentRectForFrameRect and
 // frameRectForContentRect, but they deal with the rect of the window's "main
@@ -71,8 +79,9 @@ enum class NativeKeyBindingsType : uint8_t;
 - (void)mouseEntered:(NSEvent*)aEvent;
 - (void)mouseExited:(NSEvent*)aEvent;
 - (void)mouseMoved:(NSEvent*)aEvent;
-- (void)updateTrackingArea;
 - (NSView*)trackingAreaView;
+- (void)createTrackingArea;
+- (void)removeTrackingArea;
 
 - (void)setBeingShown:(BOOL)aValue;
 - (BOOL)isBeingShown;
@@ -185,18 +194,16 @@ enum class NativeKeyBindingsType : uint8_t;
 
 class nsCocoaWindow final : public nsBaseWidget {
  private:
+  friend class nsChildView;
   typedef nsBaseWidget Inherited;
 
  public:
   nsCocoaWindow();
 
-  [[nodiscard]] nsresult Create(nsIWidget* aParent,
-                                nsNativeWidget aNativeParent,
-                                const DesktopIntRect& aRect,
+  [[nodiscard]] nsresult Create(nsIWidget* aParent, const DesktopIntRect& aRect,
                                 InitData* = nullptr) override;
 
   [[nodiscard]] nsresult Create(nsIWidget* aParent,
-                                nsNativeWidget aNativeParent,
                                 const LayoutDeviceIntRect& aRect,
                                 InitData* = nullptr) override;
 
@@ -213,7 +220,10 @@ class nsCocoaWindow final : public nsBaseWidget {
   void SetFocus(Raise, mozilla::dom::CallerType aCallerType) override;
   LayoutDeviceIntPoint WidgetToScreenOffset() override;
   LayoutDeviceIntPoint GetClientOffset() override;
-  LayoutDeviceIntMargin ClientToWindowMargin() override;
+  LayoutDeviceIntMargin NormalSizeModeClientToWindowMargin() override;
+  bool ShowsResizeIndicator(LayoutDeviceIntRect* aResizerRect) override {
+    return false;
+  }
 
   void* GetNativeData(uint32_t aDataType) override;
 
@@ -226,6 +236,104 @@ class nsCocoaWindow final : public nsBaseWidget {
   void MoveToWorkspace(const nsAString& workspaceID) override;
   void SuppressAnimation(bool aSuppress) override;
   void HideWindowChrome(bool aShouldHide) override;
+  NSView<mozView>* GetEditorView();
+
+  nsresult SynthesizeNativeKeyEvent(int32_t aNativeKeyboardLayout,
+                                    int32_t aNativeKeyCode,
+                                    uint32_t aModifierFlags,
+                                    const nsAString& aCharacters,
+                                    const nsAString& aUnmodifiedCharacters,
+                                    nsIObserver* aObserver) override;
+
+  nsresult SynthesizeNativeMouseEvent(LayoutDeviceIntPoint aPoint,
+                                      NativeMouseMessage aNativeMessage,
+                                      mozilla::MouseButton aButton,
+                                      nsIWidget::Modifiers aModifierFlags,
+                                      nsIObserver* aObserver) override;
+
+  nsresult SynthesizeNativeMouseMove(LayoutDeviceIntPoint aPoint,
+                                     nsIObserver* aObserver) override {
+    return SynthesizeNativeMouseEvent(
+        aPoint, NativeMouseMessage::Move, mozilla::MouseButton::eNotPressed,
+        nsIWidget::Modifiers::NO_MODIFIERS, aObserver);
+  }
+  nsresult SynthesizeNativeMouseScrollEvent(
+      LayoutDeviceIntPoint aPoint, uint32_t aNativeMessage, double aDeltaX,
+      double aDeltaY, double aDeltaZ, uint32_t aModifierFlags,
+      uint32_t aAdditionalFlags, nsIObserver* aObserver) override;
+  nsresult SynthesizeNativeTouchPoint(uint32_t aPointerId,
+                                      TouchPointerState aPointerState,
+                                      LayoutDeviceIntPoint aPoint,
+                                      double aPointerPressure,
+                                      uint32_t aPointerOrientation,
+                                      nsIObserver* aObserver) override;
+
+  nsresult SynthesizeNativeTouchpadDoubleTap(LayoutDeviceIntPoint aPoint,
+                                             uint32_t aModifierFlags) override;
+
+  static bool DoHasPendingInputEvent();
+  static uint32_t GetCurrentInputEventCount();
+  static void UpdateCurrentInputEventCount();
+
+  // unit conversion convenience functions
+  int32_t CocoaPointsToDevPixels(CGFloat aPts) const {
+    return nsCocoaUtils::CocoaPointsToDevPixels(aPts, BackingScaleFactor());
+  }
+  LayoutDeviceIntPoint CocoaPointsToDevPixels(const NSPoint& aPt) const {
+    return nsCocoaUtils::CocoaPointsToDevPixels(aPt, BackingScaleFactor());
+  }
+  LayoutDeviceIntPoint CocoaPointsToDevPixelsRoundDown(
+      const NSPoint& aPt) const {
+    return nsCocoaUtils::CocoaPointsToDevPixelsRoundDown(aPt,
+                                                         BackingScaleFactor());
+  }
+  LayoutDeviceIntRect CocoaPointsToDevPixels(const NSRect& aRect) const {
+    return nsCocoaUtils::CocoaPointsToDevPixels(aRect, BackingScaleFactor());
+  }
+  CGFloat DevPixelsToCocoaPoints(int32_t aPixels) const {
+    return nsCocoaUtils::DevPixelsToCocoaPoints(aPixels, BackingScaleFactor());
+  }
+  NSRect DevPixelsToCocoaPoints(const LayoutDeviceIntRect& aRect) const {
+    return nsCocoaUtils::DevPixelsToCocoaPoints(aRect, BackingScaleFactor());
+  }
+
+  // Called by nsCocoaWindow when the window's fullscreen state changes.
+  void UpdateFullscreen(bool aFullscreen);
+
+  void DispatchAPZWheelInputEvent(mozilla::InputData& aEvent);
+  nsEventStatus DispatchAPZInputEvent(mozilla::InputData& aEvent);
+
+  void DispatchDoubleTapGesture(mozilla::TimeStamp aEventTimeStamp,
+                                LayoutDeviceIntPoint aScreenPosition,
+                                mozilla::Modifiers aModifiers);
+
+  // Called when the main thread enters a phase during which visual changes
+  // are imminent and any layer updates on the compositor thread would interfere
+  // with visual atomicity.
+  // "Async" CATransactions are CATransactions which happen on a thread that's
+  // not the main thread.
+  void SuspendAsyncCATransactions();
+
+  // Called when we know that the current main thread paint will be completed
+  // once the main thread goes back to the event loop.
+  void MaybeScheduleUnsuspendAsyncCATransactions();
+
+  // Called from the runnable dispatched by
+  // MaybeScheduleUnsuspendAsyncCATransactions(). At this point we know that the
+  // main thread is done handling the visual change (such as a window resize)
+  // and we can start modifying CALayers from the compositor thread again.
+  void UnsuspendAsyncCATransactions();
+
+#ifdef DEBUG
+  // test only.
+  nsresult SetHiDPIMode(bool aHiDPI) override;
+  nsresult RestoreHiDPIMode() override;
+#endif
+
+  void UpdateVibrancy(const nsTArray<ThemeGeometry>& aThemeGeometries);
+  mozilla::VibrancyManager& EnsureVibrancyManager();
+
+  void TearDownView();
 
   bool PrepareForFullscreenTransition(nsISupports** aData) override;
   void PerformFullscreenTransition(FullscreenTransitionStage aStage,
@@ -252,13 +360,49 @@ class nsCocoaWindow final : public nsBaseWidget {
   LayoutDeviceIntRect GetScreenBounds() override;
   void ReportMoveEvent();
   void ReportSizeEvent();
+  bool WidgetTypeSupportsAcceleration() override { return true; }
+  bool ShouldUseOffMainThreadCompositing() override;
   void SetCursor(const Cursor&) override;
 
-  CGFloat BackingScaleFactor();
+  void UpdateBoundsFromView();
+  void EnsureContentLayerForMainThreadPainting();
+
+  CGFloat BackingScaleFactor() const;
   void BackingScaleFactorChanged();
   double GetDefaultScaleInternal() override;
   int32_t RoundsWidgetCoordinatesTo() override;
 
+  // Mac specific methods
+  void WillPaintWindow();
+  bool PaintWindow(LayoutDeviceIntRegion aRegion);
+  bool PaintWindowInDrawTarget(mozilla::gfx::DrawTarget* aDT,
+                               const LayoutDeviceIntRegion& aRegion,
+                               const mozilla::gfx::IntSize& aSurfaceSize);
+
+  void PaintWindowInContentLayer();
+  void HandleMainThreadCATransaction();
+
+#ifdef ACCESSIBILITY
+  already_AddRefed<mozilla::a11y::LocalAccessible> GetDocumentAccessible();
+#endif
+
+  bool WidgetPaintsBackground() override { return true; }
+
+  bool PreRender(mozilla::widget::WidgetRenderingContext* aContext) override;
+  void PostRender(mozilla::widget::WidgetRenderingContext* aContext) override;
+  RefPtr<mozilla::layers::NativeLayerRoot> GetNativeLayerRoot() override;
+
+  void UpdateWindowDraggingRegion(
+      const LayoutDeviceIntRegion& aRegion) override;
+  LayoutDeviceIntRegion GetNonDraggableRegion() {
+    return mNonDraggableRegion.Region();
+  }
+
+  void LookUpDictionary(const nsAString& aText,
+                        const nsTArray<mozilla::FontRange>& aFontRangeArray,
+                        const bool aIsVertical,
+                        const LayoutDeviceIntPoint& aPoint) override;
+
   mozilla::DesktopToLayoutDeviceScale GetDesktopToDeviceScale() final {
     return mozilla::DesktopToLayoutDeviceScale(BackingScaleFactor());
   }
@@ -266,12 +410,20 @@ class nsCocoaWindow final : public nsBaseWidget {
   nsresult SetTitle(const nsAString& aTitle) override;
 
   void Invalidate(const LayoutDeviceIntRect& aRect) override;
-  WindowRenderer* GetWindowRenderer() override;
   nsresult DispatchEvent(mozilla::WidgetGUIEvent* aEvent,
                          nsEventStatus& aStatus) override;
   void CaptureRollupEvents(bool aDoCapture) override;
   [[nodiscard]] nsresult GetAttention(int32_t aCycleCount) override;
   bool HasPendingInputEvent() override;
+  bool SendEventToNativeMenuSystem(NSEvent* aEvent);
+  void PostHandleKeyEvent(mozilla::WidgetKeyboardEvent* aEvent) override;
+  nsresult ActivateNativeMenuItemAt(const nsAString& indexString) override;
+  nsresult ForceUpdateNativeMenuAt(const nsAString& indexString) override;
+  [[nodiscard]] nsresult GetSelectionAsPlaintext(nsAString& aResult) override;
+  TextEventDispatcherListener* GetNativeTextEventDispatcherListener() override;
+  [[nodiscard]] nsresult AttachNativeKeyEvent(
+      mozilla::WidgetKeyboardEvent& aEvent) override;
+
   TransparencyMode GetTransparencyMode() override;
   void SetTransparencyMode(TransparencyMode aMode) override;
   void SetWindowShadowStyle(mozilla::WindowShadow aStyle) override;
@@ -284,19 +436,9 @@ class nsCocoaWindow final : public nsBaseWidget {
   void SetSupportsNativeFullscreen(bool aShow) override;
   void SetWindowAnimationType(WindowAnimationType aType) override;
   void SetDrawsTitle(bool aDrawTitle) override;
-  nsresult SetNonClientMargins(const LayoutDeviceIntMargin&) override;
-  void SetDrawsInTitlebar(bool aState);
+  void SetCustomTitlebar(bool) override;
   void UpdateThemeGeometries(
       const nsTArray<ThemeGeometry>& aThemeGeometries) override;
-  nsresult SynthesizeNativeMouseEvent(LayoutDeviceIntPoint aPoint,
-                                      NativeMouseMessage aNativeMessage,
-                                      mozilla::MouseButton aButton,
-                                      nsIWidget::Modifiers aModifierFlags,
-                                      nsIObserver* aObserver) override;
-  nsresult SynthesizeNativeMouseScrollEvent(
-      LayoutDeviceIntPoint aPoint, uint32_t aNativeMessage, double aDeltaX,
-      double aDeltaY, double aDeltaZ, uint32_t aModifierFlags,
-      uint32_t aAdditionalFlags, nsIObserver* aObserver) override;
   void LockAspectRatio(bool aShouldLock) override;
 
   void DispatchSizeModeEvent();
@@ -309,14 +451,14 @@ class nsCocoaWindow final : public nsBaseWidget {
   bool HasModalDescendants() const { return mNumModalDescendants > 0; }
   bool IsModal() const { return mModal; }
 
-  NSWindow* GetCocoaWindow() { return mWindow; }
+  NSWindow* GetCocoaWindow() { return [[mWindow retain] autorelease]; }
 
   void SetMenuBar(RefPtr<nsMenuBarX>&& aMenuBar);
   nsMenuBarX* GetMenuBar();
 
   void SetInputContext(const InputContext& aContext,
                        const InputContextAction& aAction) override;
-  InputContext GetInputContext() override { return mInputContext; }
+  InputContext GetInputContext() override;
   MOZ_CAN_RUN_SCRIPT bool GetEditCommands(
       mozilla::NativeKeyBindingsType aType,
       const mozilla::WidgetKeyboardEvent& aEvent,
@@ -326,14 +468,6 @@ class nsCocoaWindow final : public nsBaseWidget {
 
   bool InFullScreenMode() const { return mInFullScreenMode; }
 
-  void PauseOrResumeCompositor(bool aPause) override;
-
-  bool AsyncPanZoomEnabled() const override;
-
-  bool StartAsyncAutoscroll(const ScreenPoint& aAnchorLocation,
-                            const ScrollableLayerGuid& aGuid) override;
-  void StopAsyncAutoscroll(const ScrollableLayerGuid& aGuid) override;
-
   // Class method versions of NSWindow/Delegate callbacks which need to
   // access object state.
   void CocoaWindowWillEnterFullscreen(bool aFullscreen);
@@ -342,6 +476,10 @@ class nsCocoaWindow final : public nsBaseWidget {
   void CocoaSendToplevelActivateEvents();
   void CocoaSendToplevelDeactivateEvents();
 
+  nsIWidgetListener* GetPaintListener() const {
+    return mAttachedWidgetListener ? mAttachedWidgetListener : mWidgetListener;
+  }
+
   enum class TransitionType {
     Windowed,
     Fullscreen,
@@ -364,11 +502,10 @@ class nsCocoaWindow final : public nsBaseWidget {
 
   nsresult CreateNativeWindow(const NSRect& aRect, BorderStyle aBorderStyle,
                               bool aRectIsFrameRect, bool aIsPrivateBrowsing);
-  nsresult CreatePopupContentView(const LayoutDeviceIntRect& aRect, InitData*);
   void DestroyNativeWindow();
   void UpdateBounds();
   int32_t GetWorkspaceID();
-  void SendSetZLevelEvent();
+  void MoveVisibleWindowToWorkspace(int32_t workspaceID);
 
   void DoResize(double aX, double aY, double aWidth, double aHeight,
                 bool aRepaint, bool aConstrainToCurrentScreen);
@@ -376,29 +513,53 @@ class nsCocoaWindow final : public nsBaseWidget {
   void UpdateFullscreenState(bool aFullScreen, bool aNativeMode);
   nsresult DoMakeFullScreen(bool aFullScreen, bool aUseSystemTransition);
 
-  already_AddRefed<nsIWidget> AllocateChildPopupWidget() override {
-    return nsIWidget::CreateTopLevelWindow();
-  }
-
-  nsIWidget* mParent;        // if we're a popup, this is our parent [WEAK]
-  nsIWidget* mAncestorLink;  // link to traverse ancestors [WEAK]
-  BaseWindow* mWindow;                // our cocoa window [STRONG]
-  BaseWindow* mClosedRetainedWindow;  // a second strong reference to our
+  BaseWindow* mWindow;  // our cocoa window [STRONG]
   // window upon closing it, held through our destructor. This is useful
   // to ensure that macOS run loops which reference the window will still
   // have something to point to even if they don't use proper retain and
   // release patterns.
-  WindowDelegate*
-      mDelegate;  // our delegate for processing window msgs [STRONG]
+  WindowDelegate* mDelegate =
+      nullptr;  // our delegate for processing window msgs [STRONG]
   RefPtr<nsMenuBarX> mMenuBar;
-  nsChildView*
-      mPopupContentView;  // if this is a popup, this is its content widget
+  ChildView* mChildView = nullptr;  // Cocoa content view, [STRONG]
+#ifdef ACCESSIBILITY
+  // weak ref to this childview's associated mozAccessible for speed reasons
+  // (we get queried for it *a lot* but don't want to own it)
+  nsWeakPtr mAccessible;
+#endif
+  // Held while the compositor (or WR renderer) thread is compositing.
+  // Protects from tearing down the view during compositing and from presenting
+  // half-composited layers to the screen.
+  mozilla::Mutex mCompositingLock MOZ_UNANNOTATED{"ChildViewCompositing"};
+
+  mozilla::ViewRegion mNonDraggableRegion;
+
+  // Cached value of [mChildView backingScaleFactor], to avoid sending two obj-c
+  // messages (respondsToSelector, backingScaleFactor) every time we need to
+  // use it.
+  // ** We'll need to reinitialize this if the backing resolution changes. **
+  mutable CGFloat mBackingScaleFactor;
+
+  RefPtr<mozilla::layers::NativeLayerRootCA> mNativeLayerRoot;
+
+  // In BasicLayers mode, this is the CoreAnimation layer that contains the
+  // rendering from Gecko. It is a sublayer of mNativeLayerRoot's underlying
+  // wrapper layer.
+  // Lazily created by EnsureContentLayerForMainThreadPainting().
+  RefPtr<mozilla::layers::NativeLayerCA> mContentLayer;
+  RefPtr<mozilla::layers::SurfacePoolHandle> mPoolHandle;
+
+  // In BasicLayers mode, this is the invalid region of mContentLayer.
+  LayoutDeviceIntRegion mContentLayerInvalidRegion;
+
+  mozilla::UniquePtr<mozilla::VibrancyManager> mVibrancyManager;
+
+  RefPtr<mozilla::CancelableRunnable> mUnsuspendAsyncCATransactionsRunnable;
+
   // if this is a toplevel window, and there is any ongoing fullscreen
   // transition, it is the animation object.
   NSAnimation* mFullscreenTransitionAnimation;
   mozilla::WindowShadow mShadowStyle;
-
-  CGFloat mBackingScaleFactor;
   CGFloat mAspectRatio;
 
   WindowAnimationType mAnimationType;
@@ -414,6 +575,16 @@ class nsCocoaWindow final : public nsBaseWidget {
   mozilla::Maybe<TransitionType> mTransitionCurrent;
   std::queue<TransitionType> mTransitionsPending;
 
+  static uint32_t sLastInputEventCount;
+
+  // This is used by SynthesizeNativeTouchPoint to maintain state between
+  // multiple synthesized points
+  mozilla::UniquePtr<mozilla::MultiTouchInput> mSynthesizedTouchInput;
+
+  // A runnable we might assign to run ProcessTransitions at a later event loop.
+  // Cancelable so we can cancel it in CancelAllTransitions(), if needed.
+  RefPtr<mozilla::CancelableRunnable> mProcessTransitionsPending;
+
   // Sometimes we add a transition that wasn't requested by a caller. We do this
   // to manage transitions between states that otherwise would be rejected by
   // Cocoa. When we do this, it's useful to know when we are handling an added
@@ -464,6 +635,12 @@ class nsCocoaWindow final : public nsBaseWidget {
   bool mWasShown = false;
 
   int32_t mNumModalDescendants = 0;
+
+  // The workspaceID to move to once the window becomes visible. A value of 0
+  // is a no-op.
+  int32_t mDeferredWorkspaceID = 0;
+
+  RefPtr<mozilla::widget::TextInputHandler> mTextInputHandler;
   InputContext mInputContext;
   NSWindowAnimationBehavior mWindowAnimationBehavior;
 
