@@ -20,7 +20,7 @@
 #include "mozilla/dom/ContentChild.h"
 #include "mozilla/Services.h"
 #include "mozilla/Preferences.h"
-#include "mozilla/Telemetry.h"
+#include "mozilla/glean/WidgetMetrics.h"
 #include <algorithm>
 
 #ifdef MOZ_WIDGET_ANDROID
@@ -396,9 +396,9 @@ nsUserIdleService::nsUserIdleService()
   nsCOMPtr<nsIAsyncShutdownService> svc = services::GetAsyncShutdownService();
   MOZ_ASSERT(svc);
   nsCOMPtr<nsIAsyncShutdownClient> client;
-  auto rv = svc->GetQuitApplicationGranted(getter_AddRefs(client));
+  auto rv = svc->GetAppShutdownConfirmed(getter_AddRefs(client));
   if (NS_FAILED(rv)) {
-    // quitApplicationGranted can be undefined in some environments.
+    // appShutdownConfirmed can be undefined in some environments.
     rv = svc->GetXpcomWillShutdown(getter_AddRefs(client));
   }
   MOZ_ASSERT(NS_SUCCEEDED(rv));
@@ -739,7 +739,7 @@ void nsUserIdleService::IdleTimerCallback(void) {
   }
 
   // Tell expired listeners they are expired,and find the next timeout
-  Telemetry::AutoTimer<Telemetry::IDLE_NOTIFY_IDLE_MS> timer;
+  auto timer = glean::widget::notify_idle.Measure();
 
   // We need to initialise the time to the next idle switch.
   mDeltaToNextIdleSwitchInS = UINT32_MAX;
