@@ -4,7 +4,10 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include <windows.h>
+#include <winreg.h>
 #include <wrl.h>
+#include <powerbase.h>
+#include <cfgmgr32.h>
 
 #include "nsServiceManagerUtils.h"
 
@@ -29,6 +32,11 @@
 #include "nsPIDOMWindow.h"
 #include "nsWindowGfx.h"
 #include "Units.h"
+#include "nsWindowsHelpers.h"
+#include "WinRegistry.h"
+#include "WinUtils.h"
+
+mozilla::LazyLogModule gTabletModeLog("TabletMode");
 
 /* mingw currently doesn't support windows.ui.viewmanagement.h, so we disable it
  * until it's fixed. */
@@ -188,8 +196,10 @@ IUISettings5 : public IInspectable {
 
 using namespace mozilla;
 
+// Since Win10 and Win11 tablet modes can't both be simultaneously active, we
+// only need one backing variable for the both of them.
 enum class TabletModeState : uint8_t { Unknown, Off, On };
-static TabletModeState sInTabletModeState;
+static TabletModeState sInTabletModeState = TabletModeState::Unknown;
 
 WindowsUIUtils::WindowsUIUtils() = default;
 WindowsUIUtils::~WindowsUIUtils() = default;
@@ -225,13 +235,17 @@ WindowsUIUtils::SetWindowIcon(mozIDOMWindowProxy* aWindow,
   nsCOMPtr<nsIWidget> widget =
       nsGlobalWindowOuter::Cast(aWindow)->GetMainWidget();
   nsWindow* window = static_cast<nsWindow*>(widget.get());
+  if (!window) {
+    NS_WARNING("SetWindowIcon failed - missing widget");
+    return NS_OK;
+  }
 
   nsresult rv;
 
   if (aSmallIcon) {
     HICON hIcon = nullptr;
     rv = nsWindowGfx::CreateIcon(
-        aSmallIcon, false, mozilla::LayoutDeviceIntPoint(),
+        aSmallIcon, nullptr, false, mozilla::LayoutDeviceIntPoint(),
         nsWindowGfx::GetIconMetrics(nsWindowGfx::kSmallIcon), &hIcon);
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -241,7 +255,7 @@ WindowsUIUtils::SetWindowIcon(mozIDOMWindowProxy* aWindow,
   if (aBigIcon) {
     HICON hIcon = nullptr;
     rv = nsWindowGfx::CreateIcon(
-        aBigIcon, false, mozilla::LayoutDeviceIntPoint(),
+        aBigIcon, nullptr, false, mozilla::LayoutDeviceIntPoint(),
         nsWindowGfx::GetIconMetrics(nsWindowGfx::kRegularIcon), &hIcon);
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -259,6 +273,10 @@ WindowsUIUtils::SetWindowIconFromExe(mozIDOMWindowProxy* aWindow,
   nsCOMPtr<nsIWidget> widget =
       nsGlobalWindowOuter::Cast(aWindow)->GetMainWidget();
   nsWindow* window = static_cast<nsWindow*>(widget.get());
+  if (!window) {
+    NS_WARNING("SetWindowIconFromExe failed - missing widget");
+    return NS_OK;
+  }
 
   HICON icon = ::LoadIconW(::GetModuleHandleW(PromiseFlatString(aExe).get()),
                            MAKEINTRESOURCEW(aIndex));
@@ -275,6 +293,10 @@ WindowsUIUtils::SetWindowIconNoData(mozIDOMWindowProxy* aWindow) {
   nsCOMPtr<nsIWidget> widget =
       nsGlobalWindowOuter::Cast(aWindow)->GetMainWidget();
   nsWindow* window = static_cast<nsWindow*>(widget.get());
+  if (!window) {
+    NS_WARNING("SetWindowIconNoData failed - missing widget");
+    return NS_OK;
+  }
 
   window->SetSmallIconNoData();
   window->SetBigIconNoData();
@@ -282,17 +304,37 @@ WindowsUIUtils::SetWindowIconNoData(mozIDOMWindowProxy* aWindow) {
   return NS_OK;
 }
 
-bool WindowsUIUtils::GetInTabletMode() {
+bool WindowsUIUtils::GetInWin10TabletMode() {
   MOZ_DIAGNOSTIC_ASSERT(NS_IsMainThread());
+  if (IsWin11OrLater()) {
+    return false;
+  }
   if (sInTabletModeState == TabletModeState::Unknown) {
-    UpdateInTabletMode();
+    UpdateInWin10TabletMode();
+  }
+  return sInTabletModeState == TabletModeState::On;
+}
+
+bool WindowsUIUtils::GetInWin11TabletMode() {
+  MOZ_DIAGNOSTIC_ASSERT(NS_IsMainThread());
+  if (!IsWin11OrLater()) {
+    return false;
+  }
+  if (sInTabletModeState == TabletModeState::Unknown) {
+    UpdateInWin11TabletMode();
   }
   return sInTabletModeState == TabletModeState::On;
 }
 
 NS_IMETHODIMP
-WindowsUIUtils::GetInTabletMode(bool* aResult) {
-  *aResult = GetInTabletMode();
+WindowsUIUtils::GetInWin10TabletMode(bool* aResult) {
+  *aResult = GetInWin10TabletMode();
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+WindowsUIUtils::GetInWin11TabletMode(bool* aResult) {
+  *aResult = GetInWin11TabletMode();
   return NS_OK;
 }
 
@@ -543,7 +585,18 @@ bool WindowsUIUtils::ComputeTransparencyEffects() {
 #endif
 }
 
-void WindowsUIUtils::UpdateInTabletMode() {
+void WindowsUIUtils::UpdateInWin10TabletMode() {
+  if (IsWin11OrLater()) {
+    // (In theory we should never get here under Win11; but it's conceivable
+    // that there are third-party applications that try to "assist" legacy Win10
+    // apps by synthesizing Win10-style tablet-mode notifications.)
+    return;
+  }
+
+  // The getter below relies on querying a HWND which is affine to the main
+  // thread; its operation is not known to be thread-safe, let alone lock-free.
+  MOZ_DIAGNOSTIC_ASSERT(NS_IsMainThread());
+
 #ifndef __MINGW32__
   nsresult rv;
   nsCOMPtr<nsIWindowMediator> winMediator(
@@ -557,14 +610,7 @@ void WindowsUIUtils::UpdateInTabletMode() {
 
   rv = winMediator->GetMostRecentBrowserWindow(getter_AddRefs(navWin));
   if (NS_FAILED(rv) || !navWin) {
-    // Fall back to the hidden window
-    nsCOMPtr<nsIAppShellService> appShell(
-        do_GetService(NS_APPSHELLSERVICE_CONTRACTID));
-
-    rv = appShell->GetHiddenDOMWindow(getter_AddRefs(navWin));
-    if (NS_FAILED(rv) || !navWin) {
-      return;
-    }
+    return;
   }
 
   nsPIDOMWindowOuter* win = nsPIDOMWindowOuter::From(navWin);
@@ -599,17 +645,245 @@ void WindowsUIUtils::UpdateInTabletMode() {
   TabletModeState oldTabletModeState = sInTabletModeState;
   sInTabletModeState = mode == UserInteractionMode_Touch ? TabletModeState::On
                                                          : TabletModeState::Off;
+
   if (sInTabletModeState != oldTabletModeState) {
     nsCOMPtr<nsIObserverService> observerService =
         mozilla::services::GetObserverService();
     observerService->NotifyObservers(nullptr, "tablet-mode-change",
                                      sInTabletModeState == TabletModeState::On
-                                         ? u"tablet-mode"
+                                         ? u"win10-tablet-mode"
                                          : u"normal-mode");
   }
 #endif
 }
 
+// Cache: whether this device is believed to be capable of entering tablet mode.
+//
+// Meaningful only if `IsWin11OrLater()`.
+static Maybe<bool> sIsTabletCapable = Nothing();
+
+// The UUID of a GPIO pin which indicates whether or not a convertible device is
+// currently in tablet mode. (We copy `DEFINE_GUID`'s implementation here since
+// we can't control `INITGUID`, which the canonical one is conditional on.)
+//
+// https://learn.microsoft.com/en-us/windows-hardware/drivers/gpiobtn/laptop-slate-mode-toggling-between-states
+#define MOZ_DEFINE_GUID(name, l, w1, w2, b1, b2, b3, b4, b5, b6, b7, b8) \
+  EXTERN_C const GUID DECLSPEC_SELECTANY name = {                        \
+      l, w1, w2, {b1, b2, b3, b4, b5, b6, b7, b8}}
+/* 317fc439-3f77-41c8-b09e-08ad63272aa3 */ MOZ_DEFINE_GUID(
+    MOZ_GUID_GPIOBUTTONS_LAPTOPSLATE_INTERFACE, 0x317fc439, 0x3f77, 0x41c8,
+    0xb0, 0x9e, 0x08, 0xad, 0x63, 0x27, 0x2a, 0xa3);
+
+void WindowsUIUtils::UpdateInWin11TabletMode() {
+  // The OS-level getter itself is threadsafe, but we retain the main-thread
+  // restriction to parallel the Win10 getter's (presumed) restriction.
+  MOZ_DIAGNOSTIC_ASSERT(NS_IsMainThread());
+
+  if (!IsWin11OrLater()) {
+    // We should ordinarily never reach this point in Win10 -- but there may
+    // well be some third-party application out there that synthesizes Win11-
+    // style tablet-mode notifications.
+    return;
+  }
+
+  // ***  ***  ***  WARNING: RELIANCE ON UNDOCUMENTED BEHAVIOR  ***  ***  ***
+  //
+  // Windows 10's `UserInteractionMode` API is no longer useful under Windows
+  // 11: it always returns `UserInteractionMode_Mouse`.
+  //
+  // The documented API to query whether we're in tablet mode (alt.: "slate
+  // mode") under Windows 11 is `::GetSystemMetrics(SM_CONVERTIBLESLATEMODE)`.
+  // This returns 0 if we are in slate mode and 1 otherwise... except on devices
+  // where tablet mode is unavailable (such as desktops), in which case it
+  // returns 0 unconditionally.
+  //
+  // Unfortunately, there is no documented API to determine whether
+  // `SM_CONVERTIBLESLATEMODE` is `0` because the device is currently in slate
+  // mode or because the device can never be in slate mode.
+  //
+  // As such, we follow Chromium's lead here, and attempt to determine
+  // heuristically whether that API is going to return anything sensible.
+  // (Indeed, the heuristic below is in large part taken from Chromium.)
+
+  if (sIsTabletCapable.isNothing()) {
+    bool const heuristic = ([]() -> bool {
+      // If the user has set the relevant pref to override our tablet-detection
+      // heuristics, go with that.
+      switch (StaticPrefs::widget_windows_tablet_detection_override()) {
+        case -1:
+          MOZ_LOG(gTabletModeLog, LogLevel::Info,
+                  ("TCH: override detected (-1)"));
+          return false;
+        case 1:
+          MOZ_LOG(gTabletModeLog, LogLevel::Info,
+                  ("TCH: override detected (+1)"));
+          return true;
+        default:
+          break;
+      }
+
+      // If ::GSM(SM_CONVERTIBLESLATEMODE) is _currently_ nonzero, we must be on
+      // a system that does somnething with SM_CONVERTIBLESLATEMODE, so we can
+      // trust it.
+      if (::GetSystemMetrics(SM_CONVERTIBLESLATEMODE) != 0) {
+        MOZ_LOG(gTabletModeLog, LogLevel::Info,
+                ("TCH: SM_CONVERTIBLESLATEMODE != 0"));
+        return true;
+      }
+
+      // If the device does not support touch it can't possibly be a tablet.
+      if (GetSystemMetrics(SM_MAXIMUMTOUCHES) == 0) {
+        MOZ_LOG(gTabletModeLog, LogLevel::Info,
+                ("TCH: SM_MAXIMUMTOUCHES != 0"));
+        return false;
+      }
+
+      if (MOZ_LOG_TEST(gTabletModeLog, LogLevel::Info)) {
+        [&]() -> void {
+          // Check to see if a particular registry key [1] exists, and what its
+          // value is.
+          //
+          // This is not presently considered reliable, as some
+          // non-tablet-capable devices have this registry key present, but not
+          // set to 1 -- see bug 1932775, as well as comments in Chromium [2].
+          //
+          // This is probably strictly redundant with the CONVERTIBLESLATEMODE
+          // check above, so we only even look at it if we're logging.
+          //
+          // [1] https://learn.microsoft.com/en-us/windows-hardware/customize/desktop/unattend/microsoft-windows-gpiobuttons-convertibleslatemode
+          // [2] https://source.chromium.org/chromium/chromium/src/+/main:base/win/win_util.cc;l=240;drc=5a02fc6cdee77d0a39e9c43a4c2a29bbccc88852
+          namespace Reg = mozilla::widget::WinRegistry;
+          Reg::Key key(
+              HKEY_LOCAL_MACHINE,
+              uR"(System\CurrentControlSet\Control\PriorityControl)"_ns,
+              Reg::KeyMode::QueryValue);
+          if (!key) {
+            MOZ_LOG(gTabletModeLog, LogLevel::Info,
+                    ("TCH: \"PriorityControl\" registry path not found"));
+          }
+
+          auto const valueType = key.GetValueType(u"ConvertibleSlateMode"_ns);
+          if (valueType == Reg::ValueType::None) {
+            MOZ_LOG(gTabletModeLog, LogLevel::Info,
+                    ("TCH: 'ConvertibleSlateMode' not found"));
+            return;
+          }
+
+          if (auto const val =
+                  key.GetValueAsDword(u"ConvertibleSlateMode"_ns)) {
+            MOZ_LOG(gTabletModeLog, LogLevel::Info,
+                    ("TCH: 'ConvertibleSlateMode' found; value is 0x%08" PRIX32,
+                     *val));
+          } else {
+            MOZ_LOG(gTabletModeLog, LogLevel::Info,
+                    ("TCH: 'ConvertibleSlateMode' found, but not a DWORD "
+                     "(type=0x%08" PRIX32 ")",
+                     uint32_t(valueType)));
+          }
+        }();
+      }
+
+      // If the device has this GUID mapped to a GPIO pin, it's almost certainly
+      // tablet-capable. (It's not certain whether the converse is true.)
+      //
+      // https://learn.microsoft.com/en-us/windows-hardware/design/device-experiences/continuum#designing-your-device-for-tablet-mode
+      bool const hasTabletGpioPin = [&]() {
+        ULONG size = 0;
+        GUID guid{MOZ_GUID_GPIOBUTTONS_LAPTOPSLATE_INTERFACE};
+
+        CONFIGRET const err = ::CM_Get_Device_Interface_List_SizeW(
+            &size, &guid, nullptr, CM_GET_DEVICE_INTERFACE_LIST_PRESENT);
+
+        // (The next step at this point would usually be to call the function
+        // "::CM_Get_Device_Interface_ListW()" -- but we don't care where the
+        // associated device interface is actually mapped to; we only care
+        // whether it's mapped at all.
+        //
+        // For our purposes, a zero-length null-terminated string doesn't count
+        // as "present".)
+        return err == CR_SUCCESS && size > 1;
+      }();
+      if (hasTabletGpioPin) {
+        MOZ_LOG(gTabletModeLog, LogLevel::Info,
+                ("TCH: relevant GPIO interface found"));
+        return true;
+      }
+
+      // If the device has no rotation sensor, it's _probably_ not a convertible
+      // device. (There are exceptions! See bug 1918292.)
+      AR_STATE rotation_state;
+      if (HRESULT hr = ::GetAutoRotationState(&rotation_state); !FAILED(hr)) {
+        if ((rotation_state & (AR_NOT_SUPPORTED | AR_LAPTOP | AR_NOSENSOR)) !=
+            0) {
+          MOZ_LOG(gTabletModeLog, LogLevel::Info, ("TCH: no rotation sensor"));
+          return false;
+        }
+      }
+
+      // If the device returns `PlatformRoleSlate` for its POWER_PLATFORM_ROLE,
+      // it's probably tablet-capable.
+      //
+      // The converse is known to be false; the tablet-capable Dell Inspiron 14
+      // 7445 2-in-1 returns `PlatformRoleMobile`.
+      //
+      // (Chromium checks for PlatformRoleMobile as well, but (e.g.) a Dell XPS
+      // 15 9500 also returns `PlatformRoleMobile` despite *not* being tablet-
+      // capable.)
+      POWER_PLATFORM_ROLE const role =
+          mozilla::widget::WinUtils::GetPowerPlatformRole();
+      if (role == PlatformRoleSlate) {
+        MOZ_LOG(gTabletModeLog, LogLevel::Info,
+                ("TCH: role == PlatformRoleSlate"));
+        return true;
+      }
+
+      // Without some specific indicator of tablet-capability, assume that we're
+      // tablet-incapable.
+      MOZ_LOG(gTabletModeLog, LogLevel::Info,
+              ("TCH: no indication; falling through"));
+      return false;
+    })();
+
+    MOZ_LOG(gTabletModeLog, LogLevel::Info,
+            ("tablet-capability heuristic: %s", heuristic ? "true" : "false"));
+
+    sIsTabletCapable = Some(heuristic);
+    // If we appear not to be tablet-capable, don't bother doing the check.
+    // (We also don't need to send a signal.)
+    if (!heuristic) {
+      sInTabletModeState = TabletModeState::Off;
+      return;
+    }
+  } else if (sIsTabletCapable == Some(false)) {
+    // We've been in here before, and the heuristic came back false... but
+    // somehow, we've just gotten an update for the convertible-slate-mode
+    // state.
+    //
+    // Clearly the heuristic was wrong!
+    //
+    // TODO(rkraesig): should we add telemetry to see how often this gets hit?
+    MOZ_LOG(gTabletModeLog, LogLevel::Warning,
+            ("recv'd update signal after false heuristic run; reversing"));
+    sIsTabletCapable = Some(true);
+  }
+
+  // at this point, we must be tablet-capable
+  MOZ_ASSERT(sIsTabletCapable == Some(true));
+
+  TabletModeState const oldState = sInTabletModeState;
+  bool const isTableting =
+      ::GetSystemMetrics(SM_CONVERTIBLESLATEMODE) == 0 /* [sic!] */;
+  sInTabletModeState = isTableting ? TabletModeState::On : TabletModeState::Off;
+  if (oldState != sInTabletModeState) {
+    nsCOMPtr<nsIObserverService> observerService =
+        mozilla::services::GetObserverService();
+    observerService->NotifyObservers(nullptr, "tablet-mode-change",
+                                     sInTabletModeState == TabletModeState::On
+                                         ? u"win11-tablet-mode"
+                                         : u"normal-mode");
+  }
+}
+
 #ifndef __MINGW32__
 struct HStringDeleter {
   using pointer = HSTRING;
@@ -630,7 +904,7 @@ Result<HStringUniquePtr, HRESULT> ConvertToWindowsString(
 }
 
 static Result<Ok, nsresult> RequestShare(
-    const std::function<HRESULT(IDataRequestedEventArgs* pArgs)>& aCallback) {
+    std::function<HRESULT(IDataRequestedEventArgs* pArgs)>&& aCallback) {
   HWND hwnd = GetForegroundWindow();
   if (!hwnd) {
     return Err(NS_ERROR_FAILURE);
